import pickle
import os
from .languages import get_language_weight, is_cjk

_words = None
_translate_table = str.maketrans(dict.fromkeys("!\"#$%&()*+,/:;<=>?@[\\]^_`{|}~.", " "))  # not included ' -

def detect(text, languages=[]):
    global _words

    if _words is None:
        # Initialize
        words_file = os.path.join(os.path.dirname(__file__), "data", "words.pickle")

        if not os.path.isfile(words_file):
            from .utils import compile_data
            compile_data()

        with open(words_file, "rb") as f:
            _words = pickle.load(f, encoding="utf-8")

    text = text.lower().strip()
    text = text.translate(_translate_table)
    if is_cjk(text):
        tokens = list(text)
    else:
        tokens = text.split(" ")

    lang_bins = {}
    for tok in tokens:
        if tok in _words:
            for lang in _words[tok]:
                if (not languages) or (lang in languages):
                    if not lang in lang_bins:
                        lang_bins[lang] = 1
                    else:
                        lang_bins[lang] += 1

    if len(lang_bins) == 0:
        return "en", 0

    max_value = max(lang_bins.values())
    candidates = [l for l in lang_bins if lang_bins[l] == max_value]

    if len(candidates) == 1:
        return candidates[0], 0.9
    else:
        best_lang = candidates[0]
        best_weight = get_language_weight(best_lang)
        for lang in candidates[1:]:
            weight = get_language_weight(lang)
            if weight > best_weight:
                best_lang = lang
                best_weight = weight

        return best_lang, (1 / len(candidates)) * 0.9



import os
import pickle
from .languages import get_supported_languages, tokenize

root_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))

def compile_data():
    print("Compiling database...")
    words = {}
    langs = get_supported_languages()
    for name in langs:
        code = langs[name]

        with open(os.path.join(root_dir, "dictionaries", f"{name}.txt"), "r", encoding="utf-8") as f:
            lines = [l.strip().lower() for l in f.read().split("\n")]
            for l in lines:
                tokens = tokenize(code, l.strip())
                for tok in tokens:
                    if not tok in words:
                        words[tok] = [code]
                    elif not code in words[tok]:
                        words[tok].append(code)

    print("Serializing...")

    outfile = os.path.join(root_dir, "lexilang", "data", "words.pickle")
    with open(outfile, "wb") as f:
        pickle.dump(words, f, protocol=4)
    print(outfile)



def get_supported_languages():
    return {
        'afrikaans': 'af', 
        'albanian': 'sq', 
        'arabic': 'ar',
        'basque': 'eu',
        'bengali': 'bn', 
        'bulgarian': 'bg', 
        'catalan': 'ca', 
        'chinese': 'zh', 
        'czech': 'cs', 
        'danish': 'da', 
        'dutch': 'nl', 
        'english': 'en', 
        'esperanto': 'eo', 
        'estonian': 'et', 
        'finnish': 'fi', 
        'french': 'fr', 
        'german': 'de', 
        'greek': 'el', 
        'hebrew': 'he', 
        'hindi': 'hi', 
        'hungarian': 'hu', 
        'indonesian': 'id', 
        'italian': 'it', 
        'japanese': 'ja',
        'kabyle': 'kab',
        'kazakh': 'kk', 
        'korean': 'ko', 
        'latvian': 'lv', 
        'lithuanian': 'lt', 
        'macedonian': 'mk', 
        'norwegian': 'nb',
        'occitan': 'oc',
        'polish': 'pl', 
        'portuguese': 'pt', 
        'romanian': 'ro', 
        'russian': 'ru', 
        'serbian': 'sr', 
        'slovak': 'sk', 
        'slovenian': 'sl', 
        'spanish': 'es', 
        'swedish': 'sv', 
        'thai': 'th', 
        'turkish': 'tr', 
        'ukrainian': 'uk', 
        'vietnamese': 'vi', 
        'farsi': 'fa'
    }

# https://en.wikipedia.org/wiki/List_of_languages_by_total_number_of_speakers
_weights = {
    'en': 1.456,
    'zh': 1.138,
    'hi': 0.609,
    'es': 0.559,
    'fr': 0.310,
    'ar': 0.274,
    'bn': 0.273,
    'pt': 0.264,
    'ru': 0.255,
    'ur': 0.232,
    'id': 0.199,
    'de': 0.133,
    'ja': 0.123,
    'tr': 0.09,
    'vi': 0.086,
    'it': 0.068,
    'th': 0.061,
    'ca': 0.0081,
    'kab': 0.0055,
    'eu': 0.00008,
    'oc': 0.0008,
}

def get_language_weight(code):
    return _weights.get(code, 0.001)

def tokenize(code, text):
    if code in ['zh', 'ko', 'ja']:
        return list(text)
    else:
        return text.split(" ")

def is_cjk(text):
    c = 0
    for ch in text[:5]:
        if any([start <= ord(ch) <= end for start, end in 
                [(4352, 4607), (11904, 42191), (43072, 43135), (44032, 55215), 
                 (63744, 64255), (65072, 65103), (65381, 65500), 
                 (131072, 196607)]
                ]):
            c += 1
    return c > 1 and text.count(" ") < 3



import io
import platform
import re
import setuptools

with io.open("README.md", encoding="utf-8") as fr:
    long_description = fr.read()

if __name__ == "__main__":
    setuptools.setup(
        name="LexiLang",
        version="1.0.3",
        author="Piero Toffanin",
        author_email="pt@masseranolabs.com",
        maintainer="Piero Toffanin",
        maintainer_email="pt@masseranolabs.com",
        description="Simple, fast dictionary-based language detector",
        long_description=long_description,
        long_description_content_type="text/markdown",
        license="AGPLv3",
        url="https://github.com/LibreTranslate/LexiLang",
        classifiers=[
            "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
            "Operating System :: MacOS :: MacOS X",
            "Operating System :: Microsoft :: Windows",
            "Operating System :: POSIX :: Linux",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.4",
            "Programming Language :: Python :: 3.5",
            "Programming Language :: Python :: 3.6",
            "Programming Language :: Python :: 3.7",
            "Intended Audience :: Developers",
            "Topic :: Text Processing :: Linguistic",
        ],
        packages=["lexilang"],
        package_data={"lexilang": ["data/words.pickle"]}
    )
